import { ObjectID } from "bson";
import { Collection } from "mongodb";
import { Class } from "oly";
import { AggregationCursorTransform, CursorTransform, IDocument, IObjectDocument } from "../interfaces";
export declare abstract class Repository<T extends IDocument> {
    static of<T extends IDocument>(type: Class<T>): Class<Repository<T>>;
    type: Class<T>;
    private databaseProvider;
    private json;
    private logger;
    /**
     * Get the collection name.
     */
    readonly collectionName: string;
    /**
     * Get the collection ref.
     */
    readonly collection: Collection;
    /**
     * Insert or Update a document.
     * Accept json/object.
     */
    save(raw: Partial<T>): Promise<T>;
    /**
     *
     * @param {Object} query
     * @returns {Promise<T extends IDocument>}
     */
    findOne(query?: object): Promise<T | undefined>;
    /**
     *
     * @param {string} id
     * @returns {Promise<T extends IDocument>}
     */
    findById(id: string): Promise<T | undefined>;
    /**
     *
     * @param {Object} query
     * @param {CursorTransform} cursorTransformer
     * @returns {Promise<T[]>}
     */
    find(query?: object, cursorTransformer?: CursorTransform): Promise<T[]>;
    /**
     *
     * @param {Object | Object[]} query
     * @param {AggregationCursorTransform} cursorTransformer
     * @returns {Promise<T[]>}
     */
    aggregate(query?: object | object[], cursorTransformer?: AggregationCursorTransform): Promise<T[]>;
    /**
     * Find one document by id and replace it.
     */
    replaceById(id: string, data: Partial<T>): Promise<void>;
    /**
     * Count number of matching documents in the db to a query.
     *
     * @param {Object} mongoQuery
     * @returns {Promise<number>}
     */
    count(mongoQuery?: object): Promise<number>;
    /**
     *
     * @param {string} id
     * @returns {Promise<void>}
     */
    removeById(id: string): Promise<void>;
    /**
     *
     * @returns {Promise<void>}
     */
    clear(): Promise<void>;
    /**
     *
     * @param {string | number | ObjectID} id
     * @returns {ObjectID}
     */
    castId(id: string | number | ObjectID): ObjectID;
    /**
     *
     * @param {IObjectDocument | IDocument} document
     * @returns {T}
     */
    in(document: IObjectDocument | IDocument): T;
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {T}
     */
    out(document: Partial<T>): T;
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {Promise<void>}
     */
    beforeInsert(document: Partial<T>): Promise<void>;
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {Promise<void>}
     */
    beforeUpdate(document: Partial<T>): Promise<void>;
}
