"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var bson_1 = require("bson");
var mongodb_1 = require("mongodb");
var oly_1 = require("oly");
var oly_json_1 = require("oly-json");
var MongoException_1 = require("../exceptions/MongoException");
var DatabaseProvider_1 = require("../providers/DatabaseProvider");
var Repository = /** @class */ (function () {
    function Repository() {
    }
    Repository.of = function (type) {
        return /** @class */ (function (_super) {
            tslib_1.__extends(RepositoryOf, _super);
            function RepositoryOf() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.type = type;
                return _this;
            }
            return RepositoryOf;
        }(Repository));
    };
    Object.defineProperty(Repository.prototype, "collectionName", {
        /**
         * Get the collection name.
         */
        get: function () {
            return this.constructor.name
                .toLowerCase()
                .replace("repository", "")
                + "s";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "collection", {
        /**
         * Get the collection ref.
         */
        get: function () {
            return this.databaseProvider.db.collection(this.collectionName);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Insert or Update a document.
     * Accept json/object.
     */
    Repository.prototype.save = function (raw) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var id, v, data, result, data;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!raw._id) return [3 /*break*/, 7];
                        id = raw._id;
                        v = raw._v;
                        this.logger.debug("update", raw);
                        raw = this.json.map(this.type, raw);
                        if (!(typeof raw.beforeUpdate === "function")) return [3 /*break*/, 2];
                        return [4 /*yield*/, raw.beforeUpdate()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [4 /*yield*/, this.beforeUpdate(raw)];
                    case 3:
                        _a.sent();
                        data = this.in(Object.assign({}, raw, {
                            _v: raw._v ? v + 1 : undefined,
                        }));
                        return [4 /*yield*/, this.collection.findOneAndUpdate({ _id: this.castId(id), _v: raw._v }, data)];
                    case 4:
                        result = _a.sent();
                        if (!(!result.value && raw._v)) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.collection.findOne({ _id: this.castId(id) })];
                    case 5:
                        if (_a.sent()) {
                            throw new mongodb_1.MongoError("Version outdated, document has been updated in the meantime");
                        }
                        else {
                            throw new mongodb_1.MongoError("Document with id " + raw._id + " does not exist");
                        }
                        _a.label = 6;
                    case 6: return [2 /*return*/, this.out(result.value)];
                    case 7:
                        this.logger.debug("insert", raw);
                        raw = this.json.map(this.type, raw);
                        if (!(typeof raw.beforeInsert === "function")) return [3 /*break*/, 9];
                        return [4 /*yield*/, raw.beforeInsert()];
                    case 8:
                        _a.sent();
                        _a.label = 9;
                    case 9: return [4 /*yield*/, this.beforeInsert(raw)];
                    case 10:
                        _a.sent();
                        data = this.in(Object.assign({}, raw, { _v: 1 }));
                        return [4 /*yield*/, this.collection.insertOne(data)];
                    case 11:
                        _a.sent();
                        return [2 /*return*/, this.out(data)];
                }
            });
        });
    };
    /**
     *
     * @param {Object} query
     * @returns {Promise<T extends IDocument>}
     */
    Repository.prototype.findOne = function (query) {
        if (query === void 0) { query = {}; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var match;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.logger.debug("find one", query);
                        return [4 /*yield*/, this.collection.findOne(query)];
                    case 1:
                        match = _a.sent();
                        if (!match) {
                            return [2 /*return*/];
                        }
                        return [2 /*return*/, this.out(match)];
                }
            });
        });
    };
    /**
     *
     * @param {string} id
     * @returns {Promise<T extends IDocument>}
     */
    Repository.prototype.findById = function (id) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, this.findOne({ _id: this.castId(id) })];
            });
        });
    };
    /**
     *
     * @param {Object} query
     * @param {CursorTransform} cursorTransformer
     * @returns {Promise<T[]>}
     */
    Repository.prototype.find = function (query, cursorTransformer) {
        if (query === void 0) { query = {}; }
        if (cursorTransformer === void 0) { cursorTransformer = function (c) { return c; }; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var cursor;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.logger.debug("find", query);
                        cursor = cursorTransformer(this.collection.find(query));
                        return [4 /*yield*/, cursor.toArray()];
                    case 1: return [2 /*return*/, (_a.sent()).map(function (i) { return _this.out(i); })];
                }
            });
        });
    };
    /**
     *
     * @param {Object | Object[]} query
     * @param {AggregationCursorTransform} cursorTransformer
     * @returns {Promise<T[]>}
     */
    Repository.prototype.aggregate = function (query, cursorTransformer) {
        if (query === void 0) { query = {}; }
        if (cursorTransformer === void 0) { cursorTransformer = function (c) { return c; }; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var cursor;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.logger.debug("aggregate", query);
                        cursor = cursorTransformer(this.collection.aggregate(Array.isArray(query) ? query : [query]));
                        return [4 /*yield*/, cursor.toArray()];
                    case 1: return [2 /*return*/, (_a.sent()).map(function (i) { return _this.out(i); })];
                }
            });
        });
    };
    /**
     * Find one document by id and replace it.
     */
    Repository.prototype.replaceById = function (id, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.collection.findOneAndReplace({ _id: this.castId(id) }, this.in(data))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Count number of matching documents in the db to a query.
     *
     * @param {Object} mongoQuery
     * @returns {Promise<number>}
     */
    Repository.prototype.count = function (mongoQuery) {
        if (mongoQuery === void 0) { mongoQuery = {}; }
        return this.collection.count(mongoQuery);
    };
    /**
     *
     * @param {string} id
     * @returns {Promise<void>}
     */
    Repository.prototype.removeById = function (id) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.collection.findOneAndDelete({
                            _id: this.castId(id),
                        })];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     *
     * @returns {Promise<void>}
     */
    Repository.prototype.clear = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.collection.deleteMany({})];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     *
     * @param {string | number | ObjectID} id
     * @returns {ObjectID}
     */
    Repository.prototype.castId = function (id) {
        return new bson_1.ObjectID(id);
    };
    /**
     *
     * @param {IObjectDocument | IDocument} document
     * @returns {T}
     */
    Repository.prototype.in = function (document) {
        if (document._id && document._id instanceof bson_1.ObjectID) {
            document._id = document._id.toString();
        }
        try {
            var obj = this.json.build(this.type, document);
            if (obj._id && typeof obj._id === "string") {
                obj._id = this.castId(obj._id);
            }
            return obj;
        }
        catch (e) {
            throw new MongoException_1.MongoException(e, "Document has been rejected");
        }
    };
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {T}
     */
    Repository.prototype.out = function (document) {
        if (document._id) {
            document._id = document._id.toString();
        }
        return this.json.map(this.type, document);
    };
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {Promise<void>}
     */
    Repository.prototype.beforeInsert = function (document) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    /**
     *
     * @param {Partial<T extends IDocument>} document
     * @returns {Promise<void>}
     */
    Repository.prototype.beforeUpdate = function (document) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/];
            });
        });
    };
    tslib_1.__decorate([
        oly_1.inject(),
        tslib_1.__metadata("design:type", DatabaseProvider_1.DatabaseProvider)
    ], Repository.prototype, "databaseProvider", void 0);
    tslib_1.__decorate([
        oly_1.inject(),
        tslib_1.__metadata("design:type", oly_json_1.Json)
    ], Repository.prototype, "json", void 0);
    tslib_1.__decorate([
        oly_1.inject(),
        tslib_1.__metadata("design:type", oly_1.Logger)
    ], Repository.prototype, "logger", void 0);
    return Repository;
}());
exports.Repository = Repository;
//# sourceMappingURL=Repository.js.map